----------------------------------------------------------------------------
-- Cells act as backends for GUI widgets. e.g Button has ButtonCell that
-- subclasses the ActionCell
----------------------------------------------------------------------------
local ActionCell={}
local LoveUI, super, Cell, Event, mouse

if type(module)=="function" then
	module (..., package.seeall)
end


----------------------------------------------------------------------------
-- Loads the ActionCell class, automatically called when it's required by
-- LoveUI
-- @param lib table
-- @return class
----------------------------------------------------------------------------
function ActionCell:load(lib)
	LoveUI=lib;
	Event=LoveUI:require("Event")
	Cell=LoveUI:require("Cell")
	mouse=LoveUI:include("mouse")
	super=Cell
	super:subclass(self);
	return self;
end

----------------------------------------------------------------------------
-- Initializes the actionCell objext
-- @param controlView View
-- @return ActionCell
----------------------------------------------------------------------------
function ActionCell:init(controlView, ...)
	-- e.g local o=LoveUI.Object:alloc():init();
	LoveUI.Cell.init(self, ...)
	self.controlEvents={};
	self.controlView=controlView
	return self;
end

----------------------------------------------------------------------------
-- Sets an action to be called when the controlEvent occurs. If aTarget is 
-- supplied it'll be passed into the anAction function as the 1st argument.
-- @param anAction function
-- @param controlEvent number constant
-- @param aTarget table (optional)
----------------------------------------------------------------------------
function ActionCell:setActionForEvent(anAction, controlEvent, aTarget)
	if controlEvent==nil then
		controlEvent = Event.DEFAULT
	end
	
	if aTarget then
		self.controlEvents[controlEvent]=function (...) anAction(aTarget, ...) end ;
	else
		self.controlEvents[controlEvent]=anAction
	end
end

----------------------------------------------------------------------------
-- Used internally, to call appropriate actions when an event is triggered.
-- @param sender table
-- @param controlEvent number constant
-- @param ... any
----------------------------------------------------------------------------
function ActionCell:activateControlEvent(sender, controlEvent, ...)
	if self.controlEvents[controlEvent]~=nil then
		self.controlEvents[controlEvent](sender, ...);
		return true;
	end
	return false;
end

----------------------------------------------------------------------------
-- Called when the GUI widget receives a mouseUp event, to activate the
-- appropriate actions. 
-- @param theEvent Event
-- @return boolean whether an action was triggered by a controlEvent.
----------------------------------------------------------------------------
function ActionCell:mouseUp(theEvent)
	if theEvent.button==mouse.mouse_left and
		self:activateControlEvent(self.controlView, Event.LEFT_MOUSE_UP, theEvent) then
		return true
	end
	if theEvent.button==mouse.mouse_right and
		self:activateControlEvent(self.controlView, Event.RIGHT_MOUSE_UP, theEvent) then
		return true
	end
	return false;
end

----------------------------------------------------------------------------
-- Called when the GUI widget receives a mouseDown event, to activate the
-- appropriate actions. 
-- @param theEvent Event
-- @return boolean whether an action was triggered by a controlEvent.
----------------------------------------------------------------------------
function ActionCell:mouseDown(theEvent)
	if theEvent.button==mouse.mouse_left and
		self:activateControlEvent(self.controlView, Event.LEFT_MOUSE_DOWN, theEvent) then
		return true
	end
	if theEvent.button==mouse.mouse_right and
		self:activateControlEvent(self.controlView, Event.RIGHT_MOUSE_DOWN, theEvent) then
		return true
	end
	return false;
end

----------------------------------------------------------------------------
-- Called when the GUI widget receives a keyDown event, to activate the
-- appropriate actions. 
-- @param theEvent Event
-- @return boolean whether an action was triggered by a controlEvent.
----------------------------------------------------------------------------
function ActionCell:keyDown(theEvent)

		if self:activateControlEvent(self.controlView, Event.KEY_DOWN, theEvent) then
			return true
		end
		
	return false;
end

----------------------------------------------------------------------------
-- Called when the GUI widget receives a keyUp event, to activate the
-- appropriate actions. 
-- @param theEvent Event
-- @return boolean whether an action was triggered by a controlEvent.
----------------------------------------------------------------------------
function ActionCell:keyUp(theEvent)
		if self:activateControlEvent(self.controlView, Event.KEY_UP, theEvent) then
			return true
		end
	return false;
end

return ActionCell;