----------------------------------------------------------------------------
-- A Context contains a contentView, which holds an array of subviews to be displayed on the screen. These views also receive mouse and key events directed to them from the Context.
----------------------------------------------------------------------------


local LoveUI, View, Responder, Environment, MouseEvent, KeyEvent, super, geometry, system;
local graphics, keyboard, mouse;

local Context={};

if type(module)=="function" then
	module (..., package.seeall)
end

----------------------------------------------------------------------------
-- Loads the Context class, automatically called when it's required by
-- LoveUI
-- @param lib table
-- @return class
----------------------------------------------------------------------------
function Context:load(lib)
	LoveUI=lib;
	View = LoveUI:require("View");
	Responder = LoveUI:require("Responder");
	MouseEvent = LoveUI:require("MouseEvent");
	KeyEvent = LoveUI:require("KeyEvent");
	Environment = LoveUI:require("Environment");
	
	graphics = LoveUI:include("graphics");
	geometry = LoveUI:include("geometry");
	keyboard = LoveUI:include("keyboard");
	mouse = LoveUI:include("mouse");
	trigger = LoveUI:include("trigger");
	system = LoveUI:include("system");
	
	super=Responder
	super:subclass(self);
	return self;
end

----------------------------------------------------------------------------
-- Initializes the context object
-- @return object
----------------------------------------------------------------------------
function Context:init()
	-- e.g local o=LoveUI.Object:alloc():init();
	super.init(self);
	self.setting={
		mouseRepeat=300,
		keyRepeat=300,
		default={
				backgroundColor={255, 255, 255},
				textColor={0, 0, 0},
				selectColor={128, 128, 128, 64},
				foregroundColor={128, 128, 255},
				font=graphics.defaultFont()
			},
		selectColor={0, 0, 255}
		}
	self.event={
		eventNumber=0,
		lastMouseEvent=nil,
		clickCount=1,
		lastKeyEvent=nil,
		keysDown={},
		}
	
	self.contentView=View:new({0, 0, graphics.getWidth(), graphics.getHeight()});
	self.contentView.opaque=false
	self.currentKeyRepeat=self.setting.keyRepeat;
	self.firstResponder=self.contentView;
	return self;
end

----------------------------------------------------------------------------
-- Sets the contentView
-- @param view object
----------------------------------------------------------------------------
function Context:setContentView(view)
	view:setNextResponder(nil)
	self.__contentView=view;
	view.context=self;
	if self.firstResponder==nil then
		self:setFirstResponder(self.contentView)
	end
end

----------------------------------------------------------------------------
-- Gets the contentView
-- @return object
----------------------------------------------------------------------------
function Context:getContentView()
	return self.__contentView;
end

----------------------------------------------------------------------------
-- Sets the firstResponder
-- @param aResponder object
----------------------------------------------------------------------------
function Context:setFirstResponder(aResponder)
	if not self.firstResponder or self.firstResponder:resignFirstResponder() then
		if aResponder:acceptsFirstResponder() and aResponder:becomeFirstResponder() or aResponder==self.contentView then
			self.__firstResponder=aResponder
			return true
		end
	end
	return false
end

----------------------------------------------------------------------------
-- Gets the firstResponder
-- @return object
----------------------------------------------------------------------------
function Context:getFirstResponder()
	return self.__firstResponder
end

----------------------------------------------------------------------------
-- Adds subviews to the context
-- @param ... a list of subviews separated by commas
----------------------------------------------------------------------------
function Context:addSubview(...)
	self.contentView:addSubview(...)
end

----------------------------------------------------------------------------
-- Call this when your app is drawing the frame
-- @param ... passed to views
----------------------------------------------------------------------------
function Context:draw(...)
	local env=Environment:new()
	self.contentView:display(...);
	self.contentView:displaySubviews(...);
	env:restore();
end

----------------------------------------------------------------------------
-- Call this when your app is updating the frame
-- @param ... passed to views
----------------------------------------------------------------------------
function Context:update(...)
	if not self.firstResponder then
		return;
	end
	
	if self.event.lastKeyEvent~= nil and keyboard.isDown(self.event.lastKeyEvent.keyCode) then
		local time=system.time()
		if time-self.event.lastKeyEvent.time >=self.currentKeyRepeat then
				self:keypressed(self.event.lastKeyEvent.keyCode, self.event.lastKeyEvent.unicodeKeyCode);
				self.currentKeyRepeat=math.max(self.currentKeyRepeat*0.5, 50);
		end
	end
	

	graphics.push()
	graphics.translate(geometry.getOrigin(self.firstResponder.frame))
	self.firstResponder:update(...);
	self.firstResponder:updateSubviews(...);
	graphics.pop()
end

----------------------------------------------------------------------------
-- Call this when your app detects a key release. key is the keyCode of the 
-- key released.
-- @param key number
-- @param unicode [optional] number
----------------------------------------------------------------------------
function Context:keyreleased(key, unicode)
	if (type(key)=="string" and keyboard.key[key]) then
		key = keyboard.key[key];
	end
	if not unicode then
		unicode=key
	end
	local theEvent=self:triggerKeyEvent(key, unicode)
	
	self.currentKeyRepeat=self.setting.keyRepeat
	self.event.keysDown[theEvent.keyCode]=false;
	if self.lastKeyEvent~=nil and theEvent.keyCode==self.lastKeyEvent.keyCode then
		self.lastKeyEvent=nil;
	end
	if self.firstResponder~=nil then
		self.firstResponder:keyUp(theEvent);
	end
end

----------------------------------------------------------------------------
-- Call this when your app detects a key press. key is the keyCode of the 
-- key released.
-- @param key number
-- @param unicode [optional] number
----------------------------------------------------------------------------
function Context:keypressed(key, unicode)
	if (type(key)=="string" and keyboard.key[key]) then
		key = keyboard.key[key];
	end
	if not unicode then
		unicode=key
	end
	local theEvent = self:triggerKeyEvent(key, unicode)
	self.event.keysDown[theEvent.keyCode]=true;
	self.event.lastKeyEvent=theEvent;
	if self.firstResponder~=nil then
		self.firstResponder:keyDown(theEvent);
	end
end

----------------------------------------------------------------------------
-- Sets up a key event. Used by keypressed and keyreleased
-- @param key number
-- @param unicode number
----------------------------------------------------------------------------
function Context:triggerKeyEvent(key, unicode)
	if key==nil then
	end
	local mouseLocation = {mouse.getX(), mouse.getY()};
	local time=system.time()
	local context=self
	
	local keysDown={}
	for k, v in pairs(self.event.keysDown) do
		keysDown[k]=v;
	end
		
	local theEvent=KeyEvent:new(self, key, unicode, mouseLocation, keysDown, time);
	self.event.lastKeyEvent = theEvent;
	return theEvent
end

----------------------------------------------------------------------------
-- Call this when your app detects a mouse press. 1 is left, 2 is right
-- @param x number
-- @param y number
-- @param button number
----------------------------------------------------------------------------
function Context:mousepressed(x, y, button)
	
	local location={x, y};
	local time=system.time()
	local eventNumber=self.event.eventNumber
	if self.event.lastMouseEvent~=nil then
		if time-self.event.lastMouseEvent.time <= self.setting.mouseRepeat
			and self.event.lastMouseEvent.button==button and 
			geometry.equals(self.event.lastMouseEvent.mouseLocation, location) then
			self.event.clickCount=self.event.clickCount+1;
			
		else
			self.event.clickCount=1;
		end
	end
	
	local theEvent=self:triggerMouseEvent(x, y, button);
	local aView=self.firstResponder;
	if self.firstResponder:resignFirstResponder() then
		aView=self.contentView;
	end
	-- Now, find the front most view that is being hit by mouse.
	while aView~=nil do
		local subviews=aView.subviews;
		local viewmatched=false;
		if #subviews==0 then break end
		for k, v in pairs(subviews) do
			v=subviews[#subviews-k+1];
			if not v.hidden then
				if geometry.mouseInRect(theEvent.mouseLocation, self.contentView:convertRectFromView(v.frame, aView)) then
					-- theEvent's location is in contentView's coordinate system
					aView=v;
					aView:bringToFront();
					viewmatched=true;
					break;
				end
			end
		end
		if viewmatched==false then
			break
		end
	end
	
	if aView==nil then
		aView=self.contentView;
	end
	
	if self.firstResponder:resignFirstResponder() then
		if self:setFirstResponder(aView) then
			-- Forward thEvent to the responder
			aView:mouseDown(theEvent);
		end
	else
		self.firstResponder:mouseDown(theEvent);
	end
end

----------------------------------------------------------------------------
-- Call this when your app detects a mouse release. 1 is left, 2 is right
-- @param x number
-- @param y number
-- @param button number
----------------------------------------------------------------------------
function Context:mousereleased(x, y, button)
	local theEvent=self:triggerMouseEvent(x, y, button);
	if self.firstResponder~=nil then
		self.firstResponder:mouseUp(theEvent)
	end
end


----------------------------------------------------------------------------
-- Sets up a mouse event. Used by mousepressed and mousereleased
-- @param x number
-- @param y number
-- @param button number
----------------------------------------------------------------------------

function Context:triggerMouseEvent(x, y, button)
	local location={x, y};
	local time=system.time()
	local eventNumber=self.event.eventNumber
	local keysDown={}
	for k, v in pairs(self.event.keysDown) do
		keysDown[k]=v;
	end
	local theEvent=MouseEvent:new(self, button, location, keysDown, time, self.event.clickCount)
	self.event.lastMouseEvent=theEvent;
	
	self.event.eventNumber=self.event.eventNumber+1
	return theEvent
end

--[[
function Context:reclick()
	if self.lastMouseEvent~=nil then
		self:mouseEvent(self.lastMouseEvent.mouseLocation.x, self.lastMouseEvent.mouseLocation.y, self.lastMouseEvent.button, self.mouseDown);
	else
		self:mouseEvent(love.mouse.getX(), love.mouse.getY(), 1, self.mouseDown);	
	end
end
]]--


return Context;
