----------------------------------------------------------------------------
-- Control inherits View. It is inherited by Buttons and Textfields.
-- Most widgets are implemented with a "View" frontend and a "Cell" backend
-- A Control has a view and cell and makes this common to widgets inheriting
-- Control.
----------------------------------------------------------------------------
local LoveUI, super, View, ActionCell, trigger

local Control={}

if type(module)=="function" then
	module (..., package.seeall)
end

----------------------------------------------------------------------------
-- Loads the Control class, automatically called when it's required by
-- LoveUI
-- @param lib table
-- @return class
----------------------------------------------------------------------------
function Control:load(lib)
	LoveUI=lib;
	View=LoveUI:require("View");
	ActionCell=LoveUI:require("ActionCell");
	trigger=LoveUI:include("trigger");
	super=View
	super:subclass(self);
	
	trigger.setDefault(self, "font")
	return self;
end

----------------------------------------------------------------------------
-- Initializes the Control object.
-- @param frame table
-- @param cell Cell
-- @return Control
----------------------------------------------------------------------------
function Control:init(frame, cell)
	-- e.g local o=LoveUI.Object:alloc():init();
	LoveUI.View.init(self, frame);
	if cell then
		self.cellClass=cell.__class;
	end
	self.value=nil;
	self.enabled=true;
	
	self.target=nil;
	
	if cell then
		self.cell=cell;
		local tcellValue=cell.value
		trigger.bind(self.cell, "value", self, "value");
				
		cell.value=tcellValue
	end
	self.action=nil;
	return self;
end

function Control:stringValue()
	-- return value as string
	return tostring(self.value)
end

function Control:performClick(sender)
	--click at 0,0 of this control
end

function Control:setAction(anAction, forControlEvent)
	self.cell:setActionForEvent(anAction, forControlEvent);
end


----------------------------------------------------------------------------
-- Returns whether the button can be focused upon.
-- @return boolean
----------------------------------------------------------------------------
function Control:acceptsFirstResponder()
	return self.enabled;
end


----------------------------------------------------------------------------
-- This method is invoked by Context whenever mouse presses into the button.
-- It uses the ButtonCell's mousedown method to handle the mouse press.
-- @param theEvent MouseEvent
----------------------------------------------------------------------------
function Control:mouseDown(theEvent)
	if not self.cell:mouseDown(theEvent) then
		self.nextResponder:mouseDown(theEvent)
	end
end

function Control:mouseUp(theEvent)
	if not self.cell:mouseUp(theEvent) then
		self.nextResponder:mouseUp(theEvent)
	end
end

function Control:keyDown(theEvent)
	if self.cell:keyDown(theEvent) then 
		return true;
	elseif self.nextResponder:keyDown(theEvent) then
		return true
	end
	return super.keyDown(self, theEvent);
end

function Control:keyUp(theEvent)
	if self.cell:keyUp(theEvent) then 
		return true;
	elseif self.nextResponder:keyUp(theEvent) then
		return true
	end
	return super.keyUp(self, theEvent);
end


return Control;
