----------------------------------------------------------------------------
-- A Responder are objects that responds to input events. Responder is inherited
-- by View. Key events are sent to the first(focused) responder in a context. 
-- Whilst it isn't responded to, it is passed on to its superview, and to 
-- superview of that superview, etc. 
-- Mouse events are sent to the responder where the event occurred.
----------------------------------------------------------------------------
local LoveUI = nil;
local Object=nil;
local Responder={}

if type(module)=="function" then
	module (..., package.seeall)
end

local super=nil

----------------------------------------------------------------------------
-- Loads the Responder class, automatically called when it's required by
-- LoveUI
-- @param lib table
-- @return class
----------------------------------------------------------------------------
function Responder:load(lib)
	LoveUI=lib;
	Object = LoveUI:require("Object");
	super = Object
	super:subclass(self);
	return self;
end

----------------------------------------------------------------------------
-- Initializes the Responder object.
-- @return Responder
----------------------------------------------------------------------------
function Responder:init()
	return self;
end

----------------------------------------------------------------------------
-- Overridden by subclasses. Returns true when the view can be focused by
-- mouse clicks, tab, etc, return false if not. 
-- @return boolean
----------------------------------------------------------------------------
function Responder:acceptsFirstResponder()
	-- Overridden by subclasses to return true if the receiver accepts first responder status.
	return false;
	-- By default a responder object refuses to become a first responder.
end

----------------------------------------------------------------------------
-- Overridden by subclasses. This method is called when the responder is
-- becoming the first responder. return false to refuse to be the first 
-- responder. return true to accept
-- @return boolean
----------------------------------------------------------------------------
function Responder:becomeFirstResponder()
	-- Notifies the receiver that it's about to become first responder in its LoveUI.Context
	return true;
	-- Returning false refuses to be first responder. Never invoke this directly.
end

----------------------------------------------------------------------------
-- The context and other responders sends a key down event to this responder by
-- calling this method. Overridden by subclasses
-- The default implementation simply passes this message to the next responder.
-- @param theEvent Event
----------------------------------------------------------------------------
function Responder:keyDown(theEvent)
	-- The default implementation simply passes this message to the next responder.
	if self.nextResponder then
		self.nextResponder:keyDown(theEvent)
	end
end

----------------------------------------------------------------------------
-- The context and other responders sends a key up event to this responder by
-- calling this method. Overridden by subclasses
-- The default implementation simply passes this message to the next responder.
-- @param theEvent Event
----------------------------------------------------------------------------
function Responder:keyUp(theEvent)
	-- The default implementation simply passes this message to the next responder.
	if self.nextResponder then
		self.nextResponder:keyUp(theEvent)
	end
end

----------------------------------------------------------------------------
-- The context and other responders sends a mouse down event to this responder by
-- calling this method. Overridden by subclasses
-- The default implementation simply passes this message to the next responder.
-- @param theEvent Event
----------------------------------------------------------------------------
function Responder:mouseDown(theEvent)
	-- The default implementation simply passes this message to the next responder.
	if self.nextResponder then
		self.nextResponder:mouseDown(theEvent)
	end
end

----------------------------------------------------------------------------
-- The context and other responders sends a mouse up event to this responder by
-- calling this method. Overridden by subclasses
-- The default implementation simply passes this message to the next responder.
-- @param theEvent Event
----------------------------------------------------------------------------
function Responder:mouseUp(theEvent)
	-- The default implementation simply passes this message to the next responder.
	if self.nextResponder then
		self.nextResponder:mouseUp(theEvent)
	end
end

----------------------------------------------------------------------------
-- Overridden by subclasses. This method is called when the responder is
-- asked to lose its first responder status. return false to refuse to 
-- lose it. return true to accept
-- @return boolean
----------------------------------------------------------------------------
function Responder:resignFirstResponder()
	-- Notifies the receiver that it's been asked to relinquish its status as first responder in its window.
	return true;
end

----------------------------------------------------------------------------
-- This is an internal LoveUI method to insert responder into the responder
-- chaing. Do not override.
-- @param aResponder Responder
----------------------------------------------------------------------------
function Responder:setNextResponder(aResponder)
	local t=rawget(self, "nextResponder");
	rawset(self, "nextResponder", aResponder)
	if aResponder and aResponder.setNextResponder then
		aResponder:setNextResponder(t);
	end
		--inserting aResponder into responder chain.
	-- Sets the receiver's next responder.
end

return Responder

