----------------------------------------------------------------------------
-- A View can contain other views. Most GUI widgets such as button and 
-- textfields are built based on the view, the View being a superclass. 
-- View is inherited from Responder
----------------------------------------------------------------------------
local LoveUI, super, geometry, trigger, Object, graphics, Environment = nil;
local View={}
 
if type(module)=="function" then
	module (..., package.seeall)
end
----------------------------------------------------------------------------
-- Loads the View class, automatically called when it's required by
-- LoveUI
-- @param lib table
-- @return class
----------------------------------------------------------------------------
function View:load(lib)
	LoveUI=lib;
	Responder = LoveUI:require("Responder");
	Environment = LoveUI:require("Environment");
	
	geometry=LoveUI:include("geometry");
	graphics=LoveUI:include("graphics");
	keyboard=LoveUI:include("keyboard");
	trigger=LoveUI:include("trigger");
	super=Responder
	Responder:subclass(self);
	
	trigger.setDefault(self, "backgroundColor")
	trigger.setDefault(self, "textColor")
	trigger.setDefault(self, "selectColor")
	--trigger.setTrigger(self, "frame")
	self.__name="View"
	
	return self;
end

----------------------------------------------------------------------------
-- Initializes the view object. frame is a rectangle {x, y, w, h}. 
-- Coordinates x, y are relative to the view (a superview) that is going to contain
-- this view. 
-- @param frame table
-- @return object
----------------------------------------------------------------------------
function View:init(frame)
	-- e.g local o=LoveUI.Object:alloc():init();
	super.init(self);
	
	
	self.frame=frame or {-1, -1, -1, -1};
	self.scissor=frame;
	self.subviews={};
	self.superview=nil;
	self.context=nil;
	self.opaque=true;
	self.hidden=false;
	self.backgroundColor=nil--self.context.setting.default.backgroundColor
	self.textColor=nil--self.context.setting.default.textColor
	self.selectColor=nil--self.context.setting.default.selectColor
	self:calculateScissor()
	return self;
end

----------------------------------------------------------------------------
-- Overridden by subclasses. Returns true when the view can be focused by
-- mouse clicks, tab, etc, return false if not. 
-- @return boolean
----------------------------------------------------------------------------
function View:acceptsFirstResponder()
	return true;
end

----------------------------------------------------------------------------
-- Setter for view.frame. Automatically run whenever view.frame is set. 
-- Re-calculates the scissor frame.
-- @param frame table
----------------------------------------------------------------------------
function View:setFrame(frame)
	self:calculateScissor();
	self.__frame={unpack(frame)}
end


----------------------------------------------------------------------------
-- Getter for view.frame Automatically run whenever view.frame is queried. 
-- Gets the view's frame
-- @return table
----------------------------------------------------------------------------
function View:getFrame()
	return self.__frame
end

----------------------------------------------------------------------------
-- Draws background if view.opaque is set to true. Overridden by subclasses
-- for their respective drawing methods.
----------------------------------------------------------------------------
function View:display()
	if self.opaque then
		local size=geometry.getSize(self.frame);
		graphics.setColor(self.backgroundColor)
		graphics.drawBackground(size)
	end
end

--function View:convertOriginToBase()
--	local p=self.superview:convertPointToBase(geometry.getOrigin(self.frame))
--	return p.x, p.y;
--end

----------------------------------------------------------------------------
-- Internally used by LoveUI when adding views with subviews
-- Do not override.
----------------------------------------------------------------------------
function View:toSubviews(fn)
	fn(self);
	for k, v in pairs(self.subviews) do
		--if fn(v) == false then
		--	return false;
		--end
		v:toSubviews(fn);
	end
end

----------------------------------------------------------------------------
-- Overridden by subclasses for their respective update methods.
----------------------------------------------------------------------------
function View:update(dt)
--	if not self.hidden then
--	end
end


----------------------------------------------------------------------------
-- Internally used by LoveUI to update the subviews of each view.
-- Do not override.
----------------------------------------------------------------------------
function View:updateSubviews(...)
	if not self.hidden then
		for k, v in pairs(self.subviews) do
			if not v.hidden then
				graphics.push()
				graphics.translate(geometry.getOrigin(v.frame))
				v:update(...);
				v:updateSubviews(...);
				graphics.pop()
			end
		end
	end
end

----------------------------------------------------------------------------
-- Re-calculates the view's scissor frame. It is called whenever the 
-- view's frame is changed. Also re-calculates the view's subviews
-- scissor frames.
----------------------------------------------------------------------------
function View:calculateScissor()
	if self.superview then
		
		local super_frame=self:convertRectFromView(self.superview.scissor, self.superview)
		local child_frame=self:convertRectFromView(self.frame, self);
		local x1, y1, x2, y2 = math.max(super_frame[1], 0), math.max(super_frame[2], 0), 
		math.min(super_frame[3]+super_frame[1], child_frame[3]), 
		math.min(super_frame[4]+super_frame[2], child_frame[4]);
		--self.scissor=self:convertRectToView({x1, y1, x2-x1, y2-y1}, self.context.contentView);
		self.scissor={x1, y1, x2-x1, y2-y1}
		for i, v in ipairs(self.subviews) do
			v:calculateScissor()
		end
	end
end

----------------------------------------------------------------------------
-- Setter for view.scissor. Automatically called when scissor is set.
-- @param value table
----------------------------------------------------------------------------

function View:setScissor(value)
	self.__scissor=value;
end

----------------------------------------------------------------------------
-- Getter for view.scissor. Automatically called when scissor is queried.
-- @return table
----------------------------------------------------------------------------

function View:getScissor()
	if not self.__scissor then
		self:calculateScissor();
	end
	return self.__scissor
end

----------------------------------------------------------------------------
-- Internally used by LoveUI to call the display methods of the view's
-- subviews. Do not override.
----------------------------------------------------------------------------

function View:displaySubviews()
	--a View is responsible for drawing itself and subviews
	if not self.hidden then
		for k, v in pairs(self.subviews) do
			if not v.hidden then
				--local env=Environment:new()
				graphics.push()
				graphics.translate(geometry.getOrigin(v.frame))
				--v:preDisplay()
					local scissor={unpack(v.scissor)}
					local translated=graphics.getTranslate();
					scissor[1]=scissor[1]+translated[1]
					scissor[2]=scissor[2]+translated[2]
					graphics.setScissor(scissor);
					v:display();
					v:displaySubviews();
				graphics.pop()
				v:postDisplay()
				--env:restore()
			end
		end
	end
end
--
--function View:preDisplay()
--end

----------------------------------------------------------------------------
-- Alternate display function that runs in the superview's
-- perspective (i.e using the superview's scissor frames, translation, etc).
-- Primarily used to draw borders and/or selection rectangles around widgets. 
----------------------------------------------------------------------------
function View:postDisplay()
	if self.tabAccessible and self.context.firstResponder == self then
		graphics.setColor(self.context.setting.selectColor)
		graphics.drawRectangle(self.frame, 1, 1, -2, -2)
	end
end


----------------------------------------------------------------------------
-- Determines the index this view resides in the superview's subview array
-- @return number
----------------------------------------------------------------------------
function View:getIndex()
	if self.superview then
		for k, v in pairs(self.superview.subviews) do
			if v==self then
				return k
			end
		end
	end
end

----------------------------------------------------------------------------
-- Gets the previous view in the superview's subview array
-- @return View
----------------------------------------------------------------------------
function View:getPreviousView()
	if not self.superview then return nil end;
	local k=self:getIndex();
	if k-1>=1 then
		return self.superview.subviews[k-1]
	end
	return nil;
end

----------------------------------------------------------------------------
-- Gets the previous view in the tree of views
-- @return View
----------------------------------------------------------------------------
function View:getPreviousViewInHierarchy()

	if #self.subviews>=1 then
		return self.subviews[#self.subviews]
	end
	
	local previousView;
	previousView=self:getPreviousView();
	
	if previousView then
		return previousView;
	end
	
	local pSuper=self.superview
	nextView=pSuper:getPreviousView()
	while not previousView do
		pSuper=pSuper.superview
		if not pSuper then
			break;
		end
		previousView=pSuper:getPreviousView()
		if previousView then
			break;
		end
	end
	
	if previousView then
		return previousView;
	else
		return self.context.contentView
	end
	
end

----------------------------------------------------------------------------
-- Gets the next view in the superview's subview array
-- @return View
----------------------------------------------------------------------------
function View:getNextView()
	if not self.superview then return nil end;
	local k=self:getIndex();
	return self.superview.subviews[k+1]
end

----------------------------------------------------------------------------
-- Gets the next view in the tree of views
-- @return View
----------------------------------------------------------------------------
function View:getNextViewInHierarchy()

	if #self.subviews>=1 then
		return self.subviews[1]
	end
	
	local nextView;
	nextView=self:getNextView();
	
	if nextView then
		return nextView;
	end
	
	local pSuper=self.superview
	nextView=pSuper:getNextView()
	while not nextView do
		pSuper=pSuper.superview
		if not pSuper then
			break;
		end
		nextView=pSuper:getNextView()
		if nextView then
			break;
		end
	end
	
	if nextView then
		return nextView;
	else
		return self.context.contentView
	end
	
end

----------------------------------------------------------------------------
-- Selects next view in the tree of views that is tab accessible
-- returns if found another view other than itself
-- @return boolean
----------------------------------------------------------------------------
function View:getNextTabAccessControl()
	local nextView=self;--=self:getNextView();
	while true do
		nextView=nextView:getNextViewInHierarchy();
		if nextView.tabAccessible and not nextView.hidden and nextView.enabled and nextView.context and nextView:acceptsFirstResponder() then
			self.context:setFirstResponder(nextView);
			break;
		end
		if nextView==self then
			return false;
		end
	end
	
	return true
end

----------------------------------------------------------------------------
-- Selects previous view in the tree of views that is tab accessible
-- returns if found another view other than itself
-- @return boolean
----------------------------------------------------------------------------
function View:getPreviousTabAccessControl()
	local previousView=self;--=self:getNextView();
	while true do
		previousView=previousView:getPreviousViewInHierarchy();
		if previousView.tabAccessible and not previousView.hidden and previousView.enabled and previousView.context and previousView:acceptsFirstResponder() then
			self.context:setFirstResponder(previousView);
			break;
		end
		if previousView==self then
			return false;
		end
	end
	
	return true
end

----------------------------------------------------------------------------
-- Called whenever a key is pressed while this View is focused.
-- Overridden by subclasses to implement their own key press events
-- The default keydown method responds to a tab and shift+tab key press 
-- to select the next or previous view.
-- @param theEvent Event
----------------------------------------------------------------------------
function View:keyDown(theEvent)
	if theEvent.keyCode==keyboard.key_tab and (self.tabAccessible or self==self.context.contentView) then
		if theEvent.keysHeld[keyboard.key_lshift] or theEvent.keysHeld[keyboard.key_rshift] then
			self:getPreviousTabAccessControl();
		else
			self:getNextTabAccessControl();
		end
		
	else
		LoveUI.Responder.keyDown(self, theEvent);
	end
end


----------------------------------------------------------------------------
-- Removes each subview in ... that belongs to the view.
-- @param ... a list of views
----------------------------------------------------------------------------
function View:removeSubview(...)
	local views={...};
	for k, aView in pairs(views) do
		for l, v in pairs(self.subviews) do
			if v==aView then
				table.remove(self.subviews, l);
				aView.superview=nil;
				aView.context=nil;
				aView.nextResponder=nil;
				break;
			end
		end
	end
end

----------------------------------------------------------------------------
-- For each key k in t, view[k]=t[k]. Useful for changing the styles of
-- multiple widgets.
-- @param t table
----------------------------------------------------------------------------
function View:apply(t)
	for k, v in pairs(t) do
		self[k]=v;
	end
end

----------------------------------------------------------------------------
-- Adds each subview in ... to the view. Added views cannot have superviews
-- already.
-- @param ... a list of views
----------------------------------------------------------------------------
function View:addSubview(...)
	local views={...};
	for k, aView in pairs(views) do
		if aView.superview==nil then
			self.subviews[#self.subviews+1]=aView
			aView.superview=self;
			aView.context=self.context;
			aView.nextResponder=self;
			aView:toSubviews(function (v) v.context=self.context end)
			aView:calculateScissor();
		else
			LoveUI.error("View to be added already has a super view! It must be removed from its super view's subviews before being added to a view as a subview again! You can copy views by going aView:copy(), which returns a copy")
		end
	end
end

--
--function View:lastSubview()
--	local lastIndex=#self.subviews;
--	while self.subviews[lastIndex].hidden do
--		if lastIndex==0 then
--			return nil
--		end
--		lastIndex=lastIndex-1;
--	end
--	return self.subviews[lastIndex];
--end

function View:mouseInRect(aPoint, aRect)
	return geometry.mouseInRect(aPoint, aRect) and not self.hidden
end

function View:convertPointToBase(aPoint)
	--each view's origin is in its superview's coord system
	local view=self;
	local x=aPoint[1]
	local y=aPoint[2]
	while view~=nil do
		x=x+view.frame[1];
		y=y+view.frame[2];
		view=view.superview;
	end
	return {x, y}
end

function View:convertPointFromBase(aPoint)
	local view=self;
	local x=aPoint[1]
	local y=aPoint[2]
	while view~=nil do
		x=x-view.frame[1];
		y=y-view.frame[2];
		view=view.superview;
	end
	return {x, y}
end

function View:convertRectToView(aRect, aView)
	local originThatView=0;
	if aView~=nil then
		originThatView=aView:convertPointFromBase(self:convertPointToBase(geometry.getOrigin(aRect)));
	else
		originThatView=self:convertPointToBase(geometry.getOrigin(aRect));
	end
	return {originThatView[1], originThatView[2], aRect[3], aRect[4]};
end

function View:convertRectFromView(aRect, aView) -- put that view in this view's perspective
	if not aRect then
		print("LoveUI.View: ERROR, aRect is NIL!")
	end
	local originThisView=0;
	if aView~=nil then
		originThisView=self:convertPointFromBase(aView:convertPointToBase(geometry.getOrigin(aRect)));
		
	else
		originThisView=self:convertPointFromBase(geometry.getOrigin(aRect));
	end
	
	return {originThisView[1], originThisView[2], aRect[3], aRect[4]};
end

function View:bringToFront()

end

return View
