----------------------------------------------------------------------------
-- A module of graphics functions, mapped to the base API's
----------------------------------------------------------------------------
local graphics={}
if type(module)=="function" then
	module (..., package.seeall)
end

graphics.line_rough = "rough"
graphics.line_smooth = "smooth"

local translation={{0, 0}};

-- Optimization:
local lg=love.graphics

----------------------------------------------------------------------------
-- Gets the current font
-- @return userdata
----------------------------------------------------------------------------
function graphics.getFont()
	return lg.getFont();
end

-- Optimization:
graphics.getFont = lg.getFont

----------------------------------------------------------------------------
-- Gets the current line width
-- @return number
----------------------------------------------------------------------------
function graphics.getLineWidth()
	return lg.getLineWidth();
end

-- Optimization:
graphics.getLineWidth = lg.getLineWidth

function graphics.getLineStyle()
	return lg.getLineStyle();
end

-- Optimization:
graphics.getLineStyle = lg.getLineStyle

----------------------------------------------------------------------------
-- Gets the current scissor in the form of {x, y, w, h}
-- @return userdata
----------------------------------------------------------------------------
function graphics.getScissor()
	local s={lg.getScissor()};
	return s;
end

----------------------------------------------------------------------------
-- Gets the current color in the form of {r, g, b}
-- @return table
----------------------------------------------------------------------------
function graphics.getColor()
	return {lg.getColor()};
end

----------------------------------------------------------------------------
-- Gets the current text color in the form of {r, g, b}
-- @return userdata
----------------------------------------------------------------------------
function graphics.getTextColor()
	return {lg.getColor()};
end

----------------------------------------------------------------------------
-- Gets the current point size
-- @return number
----------------------------------------------------------------------------
function graphics.getPointSize()
	return lg.getPointSize();
end

-- Optimization:
graphics.getPointSize = lg.getPointSize

----------------------------------------------------------------------------
-- Gets the height of the drawing area
-- @return number
----------------------------------------------------------------------------
function graphics.getHeight()
	return lg.getHeight();
end

-- Optimization:
graphics.getHeight = lg.getHeight

----------------------------------------------------------------------------
-- Gets the width of the drawing area
-- @return number
----------------------------------------------------------------------------
function graphics.getWidth()
	return lg.getWidth();
end

-- Optimization:
graphics.getWidth = lg.getWidth

----------------------------------------------------------------------------
-- Sets the scissor box
-- @param s a rectangle in the form of {x, y, w, h}
----------------------------------------------------------------------------
function graphics.setScissor(s)
	lg.setScissor(unpack(s));
end

----------------------------------------------------------------------------
-- Sets the current color
-- @param c {r, g, b}
----------------------------------------------------------------------------
function graphics.setColor(c)
	lg.setColor(unpack(c));
end

----------------------------------------------------------------------------
-- Sets the current font
-- @param f userdata
----------------------------------------------------------------------------
function graphics.setFont(f)
	if f then
		lg.setFont(f);
	end
end

-- Optimization:
graphics.setFont = lg.setFont

----------------------------------------------------------------------------
-- Sets the current text color
-- @param c {r, g, b}
----------------------------------------------------------------------------
function graphics.setTextColor(c)
	lg.setColor(unpack(c));
end

----------------------------------------------------------------------------
-- Sets the current linewidth
-- @param c number
----------------------------------------------------------------------------
function graphics.setLineWidth(c)
	lg.setLineWidth(c);
end

-- Optimization:
graphics.setLineWidth = lg.setLineWidth

----------------------------------------------------------------------------
-- Sets the current pointsize
-- @param c number
----------------------------------------------------------------------------
function graphics.setPointSize(c)
	lg.setPointSize(c);
end

-- Optimization:
graphics.setPointSize = lg.setPointSize

function graphics.setLineStyle(c)
	lg.setLineStyle(c);
end

-- Optimization:
graphics.setLineStyle = lg.setLineStyle

----------------------------------------------------------------------------
-- Calls push matrix
----------------------------------------------------------------------------
function graphics.push()
	translation[#translation+1]={unpack(translation[#translation])}
	lg.push();
end

----------------------------------------------------------------------------
-- Calls pop matrix
----------------------------------------------------------------------------
function graphics.pop()
	translation[#translation]=nil
	lg.pop();
end

----------------------------------------------------------------------------
-- Calls translate
-- @param t {x, y}
----------------------------------------------------------------------------
function graphics.translate(t)
	translation[#translation][1]=translation[#translation][1]+t[1]
	translation[#translation][2]=translation[#translation][2]+t[2]
	lg.translate(unpack(t));
end

function graphics.getTranslate()
	return {unpack(translation[#translation])}
end

----------------------------------------------------------------------------
-- Draws a rectangle of size {w, h} at 0, 0
-- @param size {w, h}
----------------------------------------------------------------------------
function graphics.drawRectangle(rect, o1, o2, o3, o4)
	o1, o2, o3, o4= o1 or 0, o2 or 0, o3 or 0, o4 or 0
	lg.rectangle("line", rect[1]+o1, rect[2]+o2 , rect[3]+o3, rect[4]+o4) --textfield border
end

--function graphics.fillRect(rect, o1, o2, o3, o4)
--	o1, o2, o3, o4= o1 or 0, o2 or 0, o3 or 0, o4 or 0
--	return lg.rectangle(love.draw_fill, rect[1]+o1, rect[2]+o2 , rect[3]+o3, rect[4]+o4)
--end

function graphics.drawLine(o1, o2)
	lg.line(o1[1], o1[2], unpack(o2)) 
end

function graphics.drawFilledRectangle(rect, ox1, oy1, ox2, oy2)
	o1, o2, o3, o4= o1 or 0, o2 or 0, o3 or 0, o4 or 0
	lg.rectangle("fill", rect[1]+o1, rect[2]+o2 , rect[3]+o3, rect[4]+o4) 
end

function graphics.drawImage(image, rect)
	lg.draw(image, rect[1], rect[2], 0, rect[3]/image:getWidth(), rect[4]/image:getHeight());
end


function graphics.drawBackground(size)
	return lg.rectangle("fill", 0, 0, unpack(size));
end

function graphics.drawBackgroundImage(image, scaleTo)
	return lg.draw(image, 0, 0, 0, scaleTo[1]/graphics.getImageWidth(image), scaleTo[2]/graphics.getImageHeight(image));
end

function graphics.drawText(text, x, y)
	return lg.print(text, x, y);
end

function graphics.getFontHeight(font)
	return font:getHeight();
end

function graphics.getFontWidth(font, text)
	return font:getWidth(text);
end

function graphics.getImageHeight(image)
	return image:getHeight()
end

function graphics.getImageWidth(image)
	return image:getWidth()
end

----------------------------------------------------------------------------
-- Loads an image at path ...
-- @param ... string
----------------------------------------------------------------------------
function graphics.newImage(...)
	return lg.newImage(...);
end

function graphics.defaultFont()
	return lg.newFont(love._vera_ttf, 12)
end


return graphics