----------------------------------------------------------------------------
-- A module of functions used to modify the functionality of classes
-- by editing their meta-functions. 
-- Implements fallbacks, setters, getters and bindings for LoveUI classes.
----------------------------------------------------------------------------
local set, get, bindings={}, {}, {}

setmetatable(set, set)
setmetatable(get, get)

----------------------------------------------------------------------------
-- When set is referenced, this function is run. It converts a key 
-- into a setter function name
-- @param key string
-- @return string
----------------------------------------------------------------------------
function set:__index(key)
	local s="set"..string.upper(string.char(key:byte(1)))..key:sub(2);
	--self[key]=s
	rawset(self, key, s)
	return s
end

----------------------------------------------------------------------------
-- When get is referenced, this function is run. It converts a key 
-- into a getter function name
-- @param key string
-- @return string
----------------------------------------------------------------------------
function get:__index(key)
	local g="get"..string.upper(string.char(key:byte(1)))..key:sub(2);
	rawset(self, key, g)
	return g
end

local trigger={}

if type(module)=="function" then
	module (..., package.seeall)
end

----------------------------------------------------------------------------
-- Sets up triggering for a class. This is called in the class's load 
-- function
-- @param class class
----------------------------------------------------------------------------
function trigger.setup(class)

	function class:__newindex(key, value, __self)
		__self = __self or self
		if type(key)~="string" or not getmetatable(self)[set[key]] then
			return rawset(self, key, value)
		end
		
		self.__class[set[key]](self, value);
	end
	
	function class:__index(key, __self)
		__self = __self or self
		if type(key)~="string" or not getmetatable(self)[get[key]] then
			return getmetatable(self)[key]
		end
		
		return self.__class[get[key]](__self);
	end
	
end

----------------------------------------------------------------------------
-- Sets a key to revert to the context's default value if it is nil. 
-- defaultKey is the key's name inside self.context.setting.default
-- Called in the class's load function
-- @param class class
-- @param key string
-- @param defaultKey [optional] string
----------------------------------------------------------------------------
function trigger.setDefault(class, key, defaultKey)
	defaultKey=defaultKey or key
	class[key]=nil
	class[get[key]]= function (self)
		if self.context then
			return self.context.setting.default[defaultKey]
		end
		print("LoveUI.trigger: ERROR, "..self.__name.." has no context!")
		return nil;
	end
	
end

----------------------------------------------------------------------------
-- Sets a key to revert to the superview's value if it is nil. 
-- If no superview then it refers to the context's default value
-- Called in the class's load function
-- @param class class
-- @param key string
----------------------------------------------------------------------------
function trigger.setCascade(class, key)
	defaultKey=defaultKey or key
	class[get[key]]= function (self)
		if self.superview and self.superview[key] then
			return self.superview[key];
		elseif self.context then
			return self.context.setting.default[defaultKey]
		end
	end
end

----------------------------------------------------------------------------
-- Creates stub setters and getters for a given key. Called in the class's
-- load function
-- @param class class
-- @param key string
----------------------------------------------------------------------------
function trigger.setTrigger(class, key)
	class[set[key]]=function(self, key, value) self[key] = value end
	class[get[key]]=function(self, key) return self[key] end
end

----------------------------------------------------------------------------
-- Binds two object's keys together. By default index and newindex are both
-- function (v) return v end. This function can also be used for simple tables
-- if they have no metatables attached.
-- @param object1 anObject
-- @param key1 string
-- @param object2 anObject
-- @param key2 string
-- @param index [optional] function
-- @param newindex [optional] function
----------------------------------------------------------------------------
function trigger.bind(object1, key1, object2, key2, index, newindex)
	newindex=newindex or function(v) return v end
	index=index or function(v) return v end
	
	if not getmetatable(object1) then
		setmetatable(object1, {__index={}})
	end
	
	local original_mt=getmetatable(object1);
	local new_mt={}
	object1[key1]=nil;
	trigger.setup(new_mt)
	
	setmetatable(object1, new_mt)
	
	function new_mt:__index(key)
		if key==key1 then
			return object2[key2]
		end
		setmetatable(object1, original_mt)
		local r= object1[key]
		setmetatable(object1, new_mt)
		return r
	end
	
	function new_mt:__newindex (key, value)
		if key==key1 then
			object2[key2] = value
			return;
		end
		setmetatable(object1, original_mt)
		object1[key]=value
		setmetatable(object1, new_mt)
	end
		
end

return trigger